-- Create columns in alerts.status. Since v7 is dynamic we'll do it here
alter table alerts.status add XMState integer;
alter table alerts.status add XMTarget varchar(64);
alter table alerts.status add XMResponder varchar(64);
alter table alerts.status add XMLastUpdated UTC;

-- Create default xMatters users for xMatters to use when updating Netcool events
create user 'xmatters' id 99999
full name 'xMatters User'
password 'xmatters';

-- Add new user to relevant groups
alter group 'System' assign members 'xmatters';
alter group 'ISQL' assign members 'xmatters';
alter group 'ISQLWrite' assign members 'xmatters';

-- Add new Group Role
create role 'xMattersGroup';

-- create xMatters user bsmith
create group 'xMattersUsers';
create user 'bsmith' id 99998
full name 'Bob Smith'
password 'password';
alter group 'xMattersUsers' assign members 'bsmith';

-- Create the conversions for the xMatters fields
insert into alerts.conversions values ( 'XMStatus0','XMState',0,'New' );
insert into alerts.conversions values ( 'XMStatus1','XMState',1,'Marked For Dispatch' );
insert into alerts.conversions values ( 'XMStatus2','XMState',2,'Dispatched' );
insert into alerts.conversions values ( 'XMStatus3','XMState',3,'Delivered' );
insert into alerts.conversions values ( 'XMStatus4','XMState',4,'XM_Acknowleged' );
insert into alerts.conversions values ( 'XMStatus5','XMState',5,'Escalated' );
insert into alerts.conversions values ( 'XMStatus6','XMState',6,'Ignored' );
insert into alerts.conversions values ( 'XMStatus7','XMState',7,'Cancelled' );
insert into alerts.conversions values ( 'XMStatus8','XMState',8,'XM_Ignored' );
insert into alerts.conversions values ( 'XMStatus9','XMState',9,'XM_Owned' );

-- Create the Prompt for XMAssign
insert into tools.prompt_defs values ( 'xMattersAssignUsers','xMattersAssignUsersPrompt','','',7 );
insert into tools.menu_defs values ( 'xMattersAssignUsers','master','names', 'Name', 'Name', 'Name', 'UID in (select UID from master.members where GID in (select GID from master.groups where Name = \'xMattersUsers\'))' );

-- Create the Dispatch Tool
insert into tools.actions values (9999, 'Dispatch to xMatters Users', -1, 1, '', '', '', '', 1, 'update alerts.status set XMState=1, XMTarget=\'$prompt.xMattersAssignUsers\', XMResponder=\'\' where Serial in ($selected_rows.Serial); flush iduc;' , '', '', '', 0, 0, '', '', '', '', 0, 0, 1, 'NT', '', '', '', '', 0, 0 );
insert into tools.action_access values (9999, -1, -1, 9999);

-- Create the Dispatch to XM Menu
insert into tools.menu_items values ( '100:9999',100,9999,'Dispatch to xMatters Users','', 1, 0, 9999, 2 );

-- Create the Prompt for XMAssignGroup
insert into tools.prompt_defs values ( 'xMattersAssignGroup','xMattersAssignGroupPrompt','','',7 );
insert into tools.menu_defs values ( 'xMattersAssignGroup','master','groups', 'Name', 'Name', 'Name', 'GID in (select GID from master.groups where GID in (select GranteeID as GID from security.role_grants where RoleID in (select RoleID from security.roles where RoleName = \'xMattersGroup\')))' );

-- Create the Dispatch Tool
insert into tools.actions values (9998, 'Dispatch to xMatters Groups', -1, 1, '', '', '', '', 1, 'update alerts.status set XMState=1, XMTarget=\'$prompt.xMattersAssignGroup\', XMResponder=\'\' where Serial in ($selected_rows.Serial); flush iduc;' , '', '', '', 0, 0, '', '', '', '', 0, 0, 1, 'NT', '', '', '', '', 0, 0 );
insert into tools.action_access values (9998, -1, -1, 9998);

-- Create the Dispatch to XM Menu
insert into tools.menu_items values ( '100:9998',100,9998,'Dispatch to xMatters Groups','', 1, 0, 9998, 2 );

-- Create the Dispatch Tool
insert into tools.actions values (9997, 'Stop all xMatters notifications for this Event', -1, 1, '', '', '', '', 1, 'update alerts.status set XMState=7, XMTarget=\'\' where Serial in ($selected_rows.Serial); flush iduc;' , '', '', '', 0, 1, 'APClient.bin.exe "--map-data" "del" "IBMNC-"@ServerSerial', '', '', '', 0, 0, 1, 'NT', 'A manual message was submitted and the circumstance/issue has been changed.  The operator now wants to "stop" all notifications.', '', '', '', 0, 1 );
insert into tools.action_access values (9997, -1, -1, 9997);

-- Create the Dispatch to XM Menu
insert into tools.menu_items values ( '100:9997',100,9997,'Stop all xMatters notifications for this Event','', 1, 0, 9997, 2 );

-- Create a group for the xMatters triggers to go in
create or replace trigger group xmatters_triggers;
alter trigger group xmatters_triggers set enabled false;

-- Create External Action procedures to call the APClient
create or replace procedure xMattersEvent (
in XMTarget character(64),
in ServerName character(64),
in ServerSerial integer,
in Manager character(64),
in Severity integer,
in Node character(64),
in Summary character(255),
in AlertKey character(255))
executable 'Netcool-APClient.bat' host 'localhost' user 0 group 0
arguments 'NETCOOL', '\"'+XMTarget+'\"', ServerName, ServerSerial, '\"'+Manager+'\"', Severity, '\"'+Node+'\"', '\"'+Summary+'\"','\"'+AlertKey+'\"';

create or replace procedure xMattersDelEvent (
in ServerSerial integer)
executable 'Netcool-APClient.bat' host 'localhost' user 0 group 0
arguments 'del', ServerSerial;

-- Create the necessary temporal triggers for xMatters
create or replace trigger xMattersEvent
group xmatters_triggers
priority 1
comment 'Use xMatters Integration Agent to dispatch an event to xMatters'
every 15 seconds
begin

-- Select all the rows in alerts.status which should be escalated to xMatters
for each row r in alerts.status where r.XMState = 1 
  begin
  
    -- Call xMatters Integration Agent and pass the necessary ObjectServer fields to it
    execute xMattersEvent(r.XMTarget, r.ServerName, r.ServerSerial, r.Manager, r.Severity, r.Node, r.Summary, r.AlertKey);

    -- Update the alert in the ObjectServer
    update alerts.status via r.Identifier set XMState = 2, XMLastUpdated = getdate();

     -- Insert a journal entry
    insert into alerts.journal (KeyField, Serial,Chrono,Text1) values (  r.Summary + ':' + r.XMTarget + ':' + r.Identifier, r.ServerSerial, getdate(), 'xMatters: Dispatched event to xMatters for processing, targeting ' + r.XMTarget);
  end;
end;

create or replace trigger xMatters_auto_dispatch_trigger
group xmatters_triggers
priority 1
comment 'Use to automatically dispatch an event to xMatters if the Severity is Major or higher for events which are not acknowledged and have a XMState of new'
every 15 seconds
begin

-- Select all the rows in alerts.status which should be set to be dispatched to xMatters
for each row r in alerts.status where r.Severity >= 4 and r.Acknowledged = 0 and r.XMState = 0
  begin
  
    -- Update the alert in the ObjectServer
    update alerts.status via r.Identifier set XMState = 1, XMLastUpdated = getdate(), XMTarget = 'xMattersUsers';

     -- Insert a journal entry
    insert into alerts.journal (KeyField, Serial,Chrono,Text1) values (  r.Summary + ':' + r.Identifier, r.ServerSerial, getdate(), 'xMatters: Event marked for dispatch to xMatters for processing by autotrigger ');
  end;
end;

create or replace trigger xMatters_del_automated_trigger
group xmatters_triggers
priority 1
comment 'Sends a del request into xMatters when a event is deleted to stop all notifications within xMatters'
before delete on alerts.status
for each row
begin

-- Call xMatters Java Client and pass the necessary ObjectServer fields to it
execute xMattersDelEvent(old.ServerSerial);

end;

-- Activate the xMatters automations
alter trigger group xmatters_triggers set enabled true;

go